
# This file was generated by 'versioneer.py' (0.21) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2021-12-16T17:07:57+0100",
 "dirty": false,
 "error": null,
 "full-revisionid": "d4096a016b10d67fb5fd06308d1b7bc530f8f802",
 "version": "1.0b1"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
